USE [igops]
GO

DROP VIEW IF EXISTS access_review_details_v
DROP VIEW IF EXISTS adhoc_suser_v
DROP VIEW IF EXISTS suser_spermission_cert_risk_v
DROP VIEW IF EXISTS adhoc_apps_spermissions_v
DROP VIEW IF EXISTS review_item_status_v
DROP VIEW IF EXISTS shared_accounts_v
DROP VIEW IF EXISTS suser_v
DROP VIEW IF EXISTS adhoc_sgroup_suser_v
GO

CREATE VIEW shared_accounts_v AS
  SELECT 
    acct.unique_account_id,
    saccu.suser,
    suser.display_name,
    SUBSTRING(suser.dn, 4, CHARINDEX(',', suser.dn)-4) user_cn
  FROM saccount acct
    JOIN saccount_user saccu ON acct.collected_entity = saccu.saccount AND acct.relation_to_user_type = 'SHARED' AND acct.effective = '1'
    JOIN suser suser ON suser.collected_entity = saccu.suser AND suser.effective = '1' AND suser.deleted = '0' AND saccu.effective = '1'
GO

CREATE VIEW review_item_status_v AS
  SELECT 
    perm.id perm_id,
    suser.id user_id,
    item.status review_status,
    MAX(item.end_date) AS last_certification
  FROM spermission perm
    JOIN (SELECT
        item.unique_permission_id,
        item.unique_user_id,
        review_instance.id,
        review_instance.status,
        review_instance.end_date
      FROM review_item item 
        JOIN review_instance ON item.review_instance_id = review_instance.id AND review_instance.status IN ('CERTIFIED', 'CERTIFYING', 'AUDIT_APPROVAL')
      WHERE item.unique_permission_id IS NOT NULL AND item.unique_user_id IS NOT NULL) item ON item.unique_permission_id = perm.unique_permission_id AND perm.effective = 1
    JOIN suser ON suser.unique_user_id = item.unique_user_id AND suser.effective = 1
  GROUP BY perm.id, suser.id, item.status
GO

CREATE VIEW adhoc_apps_spermissions_v AS
  SELECT 
    suser_account_permission.suser,
    suser_account_permission.saccount,
    data_source.id application_id,
    data_source.name application_name,
    data_source.description application_description,
    application.risk application_risk,
    spermission.id permission_id,
    spermission.name permission_name,
    spermission.description permission_description,
    spermission.risk permission_risk,
    spermission.type permission_type,
    certified.last_certification,
      CASE
        WHEN certified.last_certification IS NOT NULL THEN 1
        ELSE 0
      END AS is_certified,
      CASE
        WHEN certified.review_status IN ('CERTIFYING', 'AUDIT_APPROVAL') THEN 1
        ELSE 0
      END AS certification_pending
  FROM suser_account_permission
    JOIN spermission ON suser_account_permission.spermission = spermission.id AND spermission.deleted = 0 AND spermission.effective = 1
    JOIN application ON spermission.application = application.id
    JOIN data_source ON application.id = data_source.id AND data_source.data_source_type = 'APPLICATION' AND data_source.deleted = 0
    LEFT JOIN review_item_status_v certified ON suser_account_permission.suser = certified.user_id AND spermission.id = certified.perm_id
GO

CREATE VIEW suser_spermission_cert_risk_v AS
  SELECT 
    spermissions.suser AS suser_id, 
    COALESCE(spermissions.is_certified, 0) AS is_certified, 
    COALESCE(spermissions.certification_pending, 0) AS certification_pending, 
    COALESCE(sum(spermissions.permission_risk), 0) AS aggregate_permission_risk, 
    COUNT(spermissions.suser) AS certification_count 
  FROM adhoc_apps_spermissions_v spermissions 
  GROUP BY spermissions.suser, spermissions.is_certified, spermissions.certification_pending, spermissions.permission_risk
GO

CREATE VIEW adhoc_suser_v AS
  SELECT
    suser.id,
    suser.deleted,
    suser.department,
    suser.display_description,
    suser.display_name,
    suser.dn,
    suser.effective,
    suser.employee_status,
    suser.employee_type,
    suser.first_name,
    suser.full_name,
    suser.guid,
    suser.home_phone,
    suser.idm_dn,
    suser.idm_tree_name,
    suser.is_curated,
    suser.job_code,
    suser.job_title,
    suser.last_name,
    suser.location,
    suser.merged,
    suser.middle_name,
    suser.office_phone,
    suser.other_phone,
    suser.preferred_locale,
    suser.provisioning_id,
    suser.secondary_supervisor_id,
    suser.supervisor_attr_key,
    suser.supervisor_id,
    suser.supervisor_unique_id,
    suser.unique_user_id,
    suser.user_id,
    suser.business_unit,
    suser.collected_entity,
    suser.effective_entity,
    suser.identity_source,
    suser.sversion,
    suser.workforce_id,
    suser_email.email,
    suser.risk,
    cert_risk.certification_count,
    cert_risk.is_certified,
    cert_risk.certification_pending,
    cert_risk.aggregate_permission_risk,
    permissions.all_permissions
  FROM suser
    LEFT JOIN suser_email ON suser.id = suser_email.suser
    LEFT JOIN (SELECT
        sap.suser,
        STUFF((SELECT ' | ' + perm_1.name FROM spermission perm_1
          JOIN suser_account_permission sap_1 ON perm_1.id = sap_1.spermission
          WHERE sap_1.suser = sap.suser
          ORDER BY perm_1.name
          FOR XML PATH('')),1,3,'') all_permissions
      FROM spermission perm
        JOIN suser_account_permission sap ON perm.id = sap.spermission
      GROUP BY sap.suser) permissions ON suser.id = permissions.suser
    LEFT JOIN suser_spermission_cert_risk_v cert_risk ON suser.id = cert_risk.suser_id
  WHERE suser.deleted = 0 AND suser.effective = 1
GO

CREATE VIEW access_review_details_v AS
  SELECT DISTINCT
    item.id review_item_id,
    item.last_override_action_id last_override_action,
    instance.id review_instance_id,
    instance.status review_status,
    instance.start_date,
    instance.end_date,
    instance.valid_to_date,
    instance.item_count,
    multi_owner.review_owners,
    multi_reviewer.reviewer_ids reviewer_id,
    CASE
      WHEN item.final_cert_by_unique_user_id IS NOT NULL THEN final_reviewer.display_name
      ELSE multi_reviewer.reviewers
    END reviewer_display_name,
    CASE
      WHEN item.final_cert_by_unique_user_id IS NOT NULL THEN  substring(final_reviewer.dn, 4,  charindex(',', final_reviewer.dn) - 4)
      ELSE multi_reviewer.user_cn
    END reviewer_cn,
    reassigned_by_history.reviewers reassigned_by_user_disp_name,
    reassigned_from_history.reviewers reassigned_from_user_disp_name,
    reassigned_by_history.reasons reassigned_reasons,
    reassigned_by_history.rev_comments reassigned_comments,
    reassigned_by_history.reassigned_date,
    defn.name review_name,
    defn.description,
    defn.review_type,
    defn.next_schedule_date,
    CASE
      WHEN item.item_type = 'ACCOUNT' AND usr.id IS NULL THEN shacc.suser  
      ELSE usr.id
    END user_id,
    CASE
      WHEN item.item_type = 'ACCOUNT' AND usr.display_name IS NULL THEN shacc.display_name
      ELSE usr.display_name
    END user_display_name,
    CASE
      WHEN item.item_type = 'ACCOUNT' AND usr.display_name IS NULL THEN shacc.user_cn
      ELSE substring(usr.dn, 4, charindex(',', usr.dn) - 4)
    END user_cn,
    action.action reviewer_action,
    reason.name modify_reason_name,
    reason.id modify_reason_id,
    CASE
      WHEN fulfillment.review_item_id IS NOT NULL THEN 1
      ELSE 0
    END remove_completed,
    usr.unique_user_id,
    action.review_order,
    action.reviewer_comment,
    perm.id permission_id,
    perm.name permission_name,
    acct.id account_id,
    acct.account_id account_native_id,
    acct.name account_name,
    acct.relation_to_user_type account_relation,
    app.id application_id,
    app_ds.name application_name,
    item.final_decision,
    item.final_decision_time,
    role.id role_id,
    role.name role_name,
    item.item_type,
    brole.name brole_name,
    action.fulfillment_instructions,
    esc_reviewer.display_name esc_reviewer
  FROM 
    review_item item
    JOIN review_instance instance ON item.review_instance_id = instance.id AND instance.status NOT IN ('PREVIEW', 'PREVIEW_CANCELED', 'PREVIEW_STARTING')
     JOIN (SELECT
          review_instance.id inst_id,
          defn.id defn_id, 
          CASE WHEN staged_defn.name IS NULL THEN defn.name ELSE staged_defn.name END AS name,
          CASE WHEN staged_defn.description IS NULL THEN defn.description ELSE staged_defn.description END AS description,
          CASE WHEN staged_defn.next_schedule_date IS NULL THEN defn.next_schedule_date ELSE staged_defn.next_schedule_date END AS next_schedule_date,
          defn.review_type,
          defn.deleted
        FROM review_definition defn
        JOIN review_instance ON defn.id = review_instance.review_definition
        LEFT JOIN review_definition staged_defn ON staged_defn.id = review_instance.staged_review_definition_id) defn ON instance.id = defn.inst_id
    LEFT JOIN suser usr ON usr.unique_user_id = item.unique_user_id AND usr.effective = 1
    LEFT JOIN spermission perm ON perm.unique_permission_id = item.unique_permission_id AND perm.effective = 1
    LEFT JOIN saccount acct ON acct.unique_account_id = item.unique_account_id AND acct.effective = 1
    LEFT JOIN application app ON app.id =
      CASE
        WHEN perm.id IS NULL THEN acct.application
        ELSE perm.application
      END
    LEFT JOIN business_role brole ON brole.id = item.business_role_id
    LEFT JOIN role_policy role ON role.id = item.role_id
    LEFT JOIN data_source app_ds ON app_ds.id = app.id
    LEFT JOIN review_item_action action ON action.review_item_id = item.id AND action.action <> 'REASSIGNED'
    LEFT JOIN review_item_action_reason reason ON reason.id = action.reason_id
    LEFT JOIN reviewer_instance inst2 ON instance.id = inst2.review_instance_id AND inst2.type = 'RO'
    LEFT JOIN reviewer_instance inst3 ON instance.id = inst3.review_instance_id
    LEFT JOIN (SELECT DISTINCT 
      item.id item_id,
      STUFF((SELECT ' | ' + CAST(multi_reviewers.id AS nvarchar(MAX)) FROM review_item item1
      LEFT JOIN reviewer_user_instance rrui ON rrui.reviewer_instance_id = item1.curr_reviewer_instance_id
      LEFT JOIN suser multi_reviewers ON rrui.unique_user_id = multi_reviewers.unique_user_id AND multi_reviewers.effective = 1
      WHERE item1.id = item.id
      ORDER BY multi_reviewers.display_name FOR XML PATH('')), 1, 3, '') reviewer_ids,
      STUFF((SELECT  ' | ' + CAST(multi_reviewers.unique_user_id AS nvarchar(MAX)) FROM review_item item1
      LEFT JOIN reviewer_user_instance rrui ON rrui.reviewer_instance_id = item1.curr_reviewer_instance_id
      LEFT JOIN suser multi_reviewers ON rrui.unique_user_id = multi_reviewers.unique_user_id AND multi_reviewers.effective = 1
      WHERE item1.id = item.id
      ORDER BY multi_reviewers.display_name FOR XML PATH('')),1,3,'') reviewer_unique_ids,
      STUFF((SELECT ' | ' + CAST(multi_reviewers.display_name AS nvarchar(MAX)) FROM review_item item1
      LEFT JOIN reviewer_user_instance rrui ON rrui.reviewer_instance_id = item1.curr_reviewer_instance_id
      LEFT JOIN suser multi_reviewers ON rrui.unique_user_id = multi_reviewers.unique_user_id AND multi_reviewers.effective = 1
      WHERE item1.id = item.id
      ORDER BY multi_reviewers.display_name FOR XML PATH('')),1,3,'') reviewers,            
      STUFF((SELECT ' | ' + CAST(substring(multi_reviewers.dn, 4, charindex(',', multi_reviewers.dn) - 4) AS nvarchar(MAX)) FROM review_item item1
      LEFT JOIN reviewer_user_instance rrui ON rrui.reviewer_instance_id = item1.curr_reviewer_instance_id
      LEFT JOIN suser multi_reviewers ON rrui.unique_user_id = multi_reviewers.unique_user_id AND multi_reviewers.effective = 1
      WHERE item1.id = item.id
      ORDER BY multi_reviewers.display_name FOR XML PATH('')),1,3, '') user_cn
      FROM review_item item
      LEFT JOIN reviewer_user_instance rrui ON rrui.reviewer_instance_id = item.curr_reviewer_instance_id
      LEFT JOIN suser multi_reviewers ON rrui.unique_user_id = multi_reviewers.unique_user_id AND multi_reviewers.effective = 1
      GROUP BY item.id) multi_reviewer ON multi_reviewer.item_id = item.id
    LEFT JOIN suser final_reviewer ON item.final_cert_by_unique_user_id = final_reviewer.unique_user_id AND final_reviewer.effective = 1
    LEFT JOIN suser acting_reviewer ON action.acted_by_user_unique_id = acting_reviewer.unique_user_id AND acting_reviewer.effective = 1
    LEFT JOIN review_item_action reassigned ON reassigned.review_item_id = item.id AND reassigned.action = 'REASSIGNED' AND reassigned.reassigned_to_user_unique_id LIKE '%' + multi_reviewer.reviewer_unique_ids + '%'
    LEFT JOIN suser curr_reviewer ON curr_reviewer.unique_user_id LIKE '%' + multi_reviewer.reviewer_unique_ids + '%' AND curr_reviewer.effective = 1
    LEFT JOIN (SELECT 
      ria.review_item_id,
      STUFF((SELECT ' | ' + usr.display_name FROM review_item_action ria_1
      LEFT JOIN reviewer_instance ri ON ri.id = ria_1.reviewer_instance_id
      LEFT JOIN suser usr ON ri.single_reviewer_unique_id = usr.unique_user_id AND usr.effective = 1
      WHERE ria_1.action = 'REASSIGNED' AND ria.review_item_id = ria_1.review_item_id
      GROUP BY ria_1.review_item_id, usr.display_name, ria_1.assigned ORDER BY ria_1.assigned FOR XML PATH('')),1,3,'') reviewers
      FROM  review_item_action ria
      LEFT JOIN reviewer_instance ri ON ri.id = ria.reviewer_instance_id
      LEFT JOIN suser usr ON ri.single_reviewer_unique_id = usr.unique_user_id AND usr.effective = 1
      WHERE ria.action = 'REASSIGNED'
      GROUP BY ria.review_item_id) reassigned_from_history ON reassigned_from_history.review_item_id = item.id
    LEFT JOIN (SELECT 
      ria.review_item_id,
      STUFF((SELECT ' | ' + usr.display_name FROM review_item_action ria_1
      LEFT JOIN suser usr ON ria_1.reassigned_by_user_unique_id = usr.unique_user_id AND usr.effective = 1
      WHERE ria_1.action = 'REASSIGNED' AND ria.review_item_id = ria_1.review_item_id ORDER BY ria_1.assigned FOR XML PATH('')),1,3,'') reviewers,
      STUFF((SELECT ' | ' + COALESCE(reason.name, 'NULL_REASON') FROM review_item_action ria_1
      LEFT JOIN review_item_action_reason reason ON ria_1.reason_id = reason.id
      LEFT JOIN suser usr ON ria_1.reassigned_by_user_unique_id = usr.unique_user_id AND usr.effective = 1
      WHERE ria_1.action = 'REASSIGNED' AND ria.review_item_id = ria_1.review_item_id ORDER BY ria_1.assigned FOR XML PATH('')),1,3,'') reasons,
      STUFF((SELECT ' | ' + COALESCE(NULLIF(ria_1.reviewer_comment, ''), 'NULL_COMMENT')  FROM  review_item_action ria_1
      LEFT JOIN suser usr ON ria_1.reassigned_by_user_unique_id = usr.unique_user_id AND usr.effective = 1
      WHERE ria_1.action = 'REASSIGNED' AND ria.review_item_id = ria_1.review_item_id ORDER BY ria_1.assigned FOR XML PATH('')),1,3,'') rev_comments,
      STUFF((SELECT ' | ' + CAST(CONVERT(DATETIMEOFFSET(0), ria_1.assigned) AS nvarchar(MAX))  FROM  review_item_action ria_1
      LEFT JOIN suser usr ON ria_1.reassigned_by_user_unique_id = usr.unique_user_id AND usr.effective = 1
      WHERE ria_1.action = 'REASSIGNED' AND ria.review_item_id = ria_1.review_item_id ORDER BY ria_1.assigned FOR XML PATH('')),1,3,'') AS reassigned_date
      FROM  review_item_action ria
      LEFT JOIN review_item_action_reason reason ON ria.reason_id = reason.id
      LEFT JOIN suser usr ON ria.reassigned_by_user_unique_id = usr.unique_user_id AND usr.effective = 1
      WHERE ria.action = 'REASSIGNED'
      GROUP BY ria.review_item_id) reassigned_by_history ON reassigned_by_history.review_item_id = item.id
    LEFT JOIN shared_accounts_v shacc ON shacc.unique_account_id = acct.unique_account_id
    LEFT JOIN ( SELECT DISTINCT defn_1.name review_name,
      ri.id rid,
      STUFF((SELECT DISTINCT ' | ' + s.display_name FROM review_instance ri1
      JOIN review_definition defn_1 ON ri1.review_definition = defn_1.id
      LEFT JOIN reviewer_instance rev_owner ON ri1.id = rev_owner.review_instance_id AND rev_owner.type = 'RO'
      LEFT JOIN reviewer_user_instance rui ON rui.reviewer_instance_id = rev_owner.id
      LEFT JOIN suser s ON rui.unique_user_id = s.unique_user_id AND s.effective = 1 
      WHERE ri1.ID = ri.id
      FOR XML PATH('')),1,3,'') review_owners
      FROM review_instance ri
      JOIN review_definition defn_1 ON ri.review_definition = defn_1.id
      LEFT JOIN reviewer_instance rev_owner ON ri.id = rev_owner.review_instance_id AND rev_owner.type = 'RO'
      LEFT JOIN reviewer_user_instance rui ON rui.reviewer_instance_id = rev_owner.id
      LEFT JOIN suser s ON rui.unique_user_id = s.unique_user_id AND s.effective = 1
      GROUP BY defn_1.name, ri.id) multi_owner ON multi_owner.rid = item.review_instance_id   
    LEFT JOIN (SELECT 
        esc_reviewers1.review_instance_id,
        STUFF((SELECT ' | ' + esc_reviewers.display_name FROM (SELECT DISTINCT
          u.display_name display_name,
          rri.review_instance_id
        FROM review_item ri
          JOIN reviewer_instance rri ON ri.review_instance_id = rri.review_instance_id AND rri.type = 'EO' 
          JOIN reviewer_user_instance rriu ON rriu.reviewer_instance_id = rri.id
          JOIN suser u ON u.unique_user_id = rriu.unique_user_id AND u.effective = 1) esc_reviewers 
        WHERE esc_reviewers.review_instance_id = esc_reviewers1.review_instance_id 
        GROUP BY esc_reviewers.review_instance_id, esc_reviewers.display_name 
        ORDER BY esc_reviewers.display_name FOR XML PATH('')),1,3,'') display_name
      FROM (SELECT DISTINCT
          u.display_name display_name,
          rri.review_instance_id
        FROM review_item ri
          JOIN reviewer_instance rri ON ri.review_instance_id = rri.review_instance_id AND rri.type = 'EO' 
          JOIN reviewer_user_instance rriu ON rriu.reviewer_instance_id = rri.id
          JOIN suser u ON u.unique_user_id = rriu.unique_user_id AND u.effective = 1) esc_reviewers1 
      GROUP BY esc_reviewers1.review_instance_id) esc_reviewer ON esc_reviewer.review_instance_id = item.review_instance_id
    LEFT JOIN ( SELECT review_item_id, action, change_item_status 
      FROM change_request_item cri
      LEFT JOIN change_item_action cia ON cri.latest_action = cia.id
      WHERE cri.change_request_type LIKE 'REMOVE' + '%' AND action = 'FULFILLED' AND (change_item_status = 'VERIFIED' OR change_item_status = 'COMPLETED')) fulfillment ON item.id = fulfillment.review_item_id
  WHERE 
    defn.deleted = 0 
    AND (acting_reviewer.unique_user_id LIKE '%' + multi_reviewer.reviewer_unique_ids + '%' OR acting_reviewer.unique_user_id IS NULL 
      AND action.review_order = 
      ( SELECT DISTINCT min(act.review_order) min
        FROM review_item_action act
        WHERE (act.action = 'PENDING' OR act.action = 'ESCALATE') AND act.review_item_id = item.id)
      OR acting_reviewer.unique_user_id IS NOT NULL AND action.shared_queue = 1) 
    AND (action.action <> 'PENDING' OR item.final_decision IS NULL)
GO

CREATE VIEW suser_v AS
  SELECT
    suser.id,
    suser.deleted,
    suser.department,
    suser.display_description,
    suser.display_name,
    suser.dn,
    suser.effective,
    suser.employee_status,
    suser.employee_type,
    suser.first_name,
    suser.full_name,
    suser.guid,
    suser.home_phone,
    suser.idm_dn,
    suser.idm_tree_name,
    suser.is_curated,
    suser.job_code,
    suser.job_title,
    suser.last_name,
    suser.location,
    suser.merged,
    suser.middle_name,
    suser.office_phone,
    suser.other_phone,
    suser.preferred_locale,
    suser.provisioning_id,
    suser.secondary_supervisor_id,
    suser.supervisor_attr_key,
    suser.supervisor_id,
    suser.supervisor_unique_id,
    suser.unique_user_id,
    suser.user_id,
    suser.business_unit,
    suser.collected_entity,
    suser.effective_entity,
    suser.identity_source,
    suser.sversion,
    suser.workforce_id,
    suser_email.email,
    suser.risk
  FROM suser
    LEFT JOIN (SELECT DISTINCT 
        suser.id suser,
        suser.display_name,
        STUFF((SELECT DISTINCT ' | ' + suser_email.email
          FROM suser suser1
            JOIN suser_email ON suser.id = suser_email.suser AND suser1.effective = 1 AND suser1.deleted = 0
          WHERE suser1.id = suser.id
          FOR XML PATH('')),1,3,'') email
      FROM suser
        JOIN suser_email ON suser.id = suser_email.suser AND suser.effective = 1 AND suser.deleted = 0
      GROUP BY suser.id, suser.display_name) suser_email ON suser.id = suser_email.suser
  WHERE suser.deleted = 0 AND suser.effective = 1
GO

CREATE VIEW adhoc_sgroup_suser_v AS
  SELECT
    sgroup_user.suser AS suser_id,
    sgroup.name AS group_name,
    sgroup.short_description,
    sgroup.id
  FROM sgroup_user
    LEFT JOIN sgroup ON sgroup_user.sgroup = sgroup.id
  WHERE sgroup.deleted = 0 AND sgroup.effective = 1
GO

GRANT SELECT ON adhoc_sgroup_suser_v TO ig_report_role
GRANT SELECT ON suser_v TO ig_report_role
GRANT SELECT ON access_review_details_v TO ig_report_role
GRANT SELECT ON shared_accounts_v TO ig_report_role
GRANT SELECT ON review_item_status_v TO ig_report_role
GRANT SELECT ON adhoc_apps_spermissions_v TO ig_report_role
GRANT SELECT ON suser_spermission_cert_risk_v TO ig_report_role
GRANT SELECT ON adhoc_suser_v TO ig_report_role
GO